/*
 *  TriMeshOpenGLViewer-Phong
 *  Copyright (C) 2005  Thanassis Tsiodras (ttsiod@softlab.ntua.gr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <GL/gl.h>

#include "Light.h"
#include "Defines.h"

int Light::_lights = 0;

void Light::set(coord x, coord y, coord z)
{
    Point::set(x,y,z);
}

void Light::SetupLightInOpenGL(Camera& camera)
{
    GLfloat ambient[]  = {AMBIENT, AMBIENT, AMBIENT, 1.0f};
    glLightfv(GL_LIGHT0 + _lightNo, GL_AMBIENT,  ambient);

    GLfloat diffuse[]  = {DIFFUSE, DIFFUSE, DIFFUSE, 1.0f};
    glLightfv(GL_LIGHT0 + _lightNo, GL_DIFFUSE,  diffuse);

    GLfloat specular[] = {1.0, 1.0, 1.0, 1.0f};
    glLightfv(GL_LIGHT0 + _lightNo, GL_SPECULAR, specular);

    GLfloat position[4];
    position[0] = getX();
    position[1] = getY();
    position[2] = getZ();
    position[3] = 1.0f;
    glLightfv(GL_LIGHT0 + _lightNo, GL_POSITION, position);

    glEnable( GL_LIGHT0 + _lightNo);
}

void Light::DrawLightUsingBitmapSun(Camera& camera)
{
    coord size=0.8;
    coord siz2=size/2.;
    glTranslatef(getX(),getY(),getZ());

    coord vLtoCx,vLtoCy,vLtoCz;

    vLtoCx = camera.getX() - getX();
    vLtoCy = camera.getY() - getY();
    vLtoCz = camera.getZ() - getZ();

    // cross with 0,0,1
    coord v2XX =0., v2YY=0., v2ZZ=1.;
    coord resXX = vLtoCy*v2ZZ - vLtoCz*v2YY;
    coord resYY = vLtoCz*v2XX - vLtoCx*v2ZZ;
    coord resZZ = vLtoCx*v2YY - vLtoCy*v2XX;
    coord v3XX = -resXX;
    coord v3YY = -resYY;
    coord v3ZZ = -resZZ;
    resXX = vLtoCy*v3ZZ - vLtoCz*v3YY;
    resYY = vLtoCz*v3XX - vLtoCx*v3ZZ;
    resZZ = vLtoCx*v3YY - vLtoCy*v3XX;

    coord norm = sqrt(v3XX*v3XX + v3YY*v3YY + v3ZZ*v3ZZ);
    v3XX/=norm; v3YY/=norm; v3ZZ/=norm;

    norm = sqrt(resXX*resXX + resYY*resYY + resZZ*resZZ);
    resXX/=norm; resYY/=norm; resZZ/=norm;

    // At this point, res contains the new Z axis
    // and v3 contains the new Y axis

    // LeftDown Point: (0,-siz2,-siz2), so in world space:
    //                    -siz2*VECTORY + -siz2*VECTORZ
    coord ldx,ldy,ldz;
    ldx = -siz2 * v3XX - siz2 * resXX;
    ldy = -siz2 * v3YY - siz2 * resYY;
    ldz = -siz2 * v3ZZ - siz2 * resZZ;

    // RightDown Point: (0,siz2,-siz2), so in world space:
    //                     siz2*VECTORY + -siz2*VECTORZ
    coord rdx,rdy,rdz;
    rdx = siz2 * v3XX - siz2 * resXX;
    rdy = siz2 * v3YY - siz2 * resYY;
    rdz = siz2 * v3ZZ - siz2 * resZZ;

    // RightUp Point: (0,siz2,siz2), so in world space:
    //                   siz2*VECTORY + siz2*VECTORZ
    coord rux,ruy,ruz;
    rux = siz2 * v3XX + siz2 * resXX;
    ruy = siz2 * v3YY + siz2 * resYY;
    ruz = siz2 * v3ZZ + siz2 * resZZ;

    // LeftUp Point: (0,-siz2,siz2), so in world space:
    //                  -siz2*VECTORY + siz2*VECTORZ
    coord lux,luy,luz;
    lux = -siz2 * v3XX + siz2 * resXX;
    luy = -siz2 * v3YY + siz2 * resYY;
    luz = -siz2 * v3ZZ + siz2 * resZZ;

    glBlendFunc(GL_ONE, GL_ONE);
    glEnable(GL_BLEND);
    glBegin(GL_QUADS);
    glColor4f(1.0, 1.0, 1.0, 0.); // alpha = semi-transparent
    glTexCoord2f(0.0f, 0.0f); glVertex3f(ldx,ldy,ldz);
    glTexCoord2f(1.0f, 0.0f); glVertex3f(rdx,rdy,rdz);
    glTexCoord2f(1.0f, 1.0f); glVertex3f(rux,ruy,ruz);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(lux,luy,luz);
    glEnd();
    glDisable(GL_BLEND);
}
