/*
 *  TriMeshOpenGLViewer-Phong
 *  Copyright (C) 2005  Thanassis Tsiodras (ttsiod@softlab.ntua.gr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __OBJECT_H__
#define __OBJECT_H__

#include <vector>
#include "Point.h"
#include "Scene.h"
#include "Camera.h"

class Vertex;
class Triangle;

class Object {
    std::vector<Vertex*> _vertexes;
    std::vector<Triangle*> _triangles;
    coord _x, _y, _z;
    coord _rx, _ry, _rz;
    Scene *_scene;
    coord _maxx, _minx, _maxy, _miny, _maxz, _minz;
    unsigned int _VBObuffers[5];
public:
    Object(const char *filename, coord x=0.0, coord y=0.0, coord z=0.0);

    void PrepareVBO();

    void setScene(Scene *scene) { _scene = scene; }

    size_t getVertexCount() { return _vertexes.size(); }

    size_t getTriangleCount() { return _triangles.size(); }

    void addVertex(Vertex *v) { _vertexes.push_back(v); }

    void move(const Point& offset) {
        _x += offset.getX();
        _y += offset.getY();
        _z += offset.getZ();
    }
    void move(coord dx, coord dy, coord dz) {
        _x+=dx; _y+=dy; _z+=dz;
    }
    void place(const Point& point) {
        _x = point.getX();
        _y = point.getY();
        _z = point.getZ();
    }
    void place(coord x, coord y, coord z) {
        _x=x; _y=y; _z=z;
    }
    void stretch(coord zoom);

    void fixNormals();

    void renderPoints(Camera&);
    void drawAALines(Camera&);
    void renderGouraud(Camera& camera);
    void renderPhong(Camera&);

    coord getX() { return _x; }
    coord getY() { return _y; }
    coord getZ() { return _z; }

    coord getMinX() { return _minx; }
    coord getMinY() { return _miny; }
    coord getMinZ() { return _minz; }
    coord getMaxX() { return _maxx; }
    coord getMaxY() { return _maxy; }
    coord getMaxZ() { return _maxz; }
    void setMinX(coord minx) { _minx = minx; }
    void setMinY(coord miny) { _miny = miny; }
    void setMinZ(coord minz) { _minz = minz; }
    void setMaxX(coord maxx) { _maxx = maxx; }
    void setMaxY(coord maxy) { _maxy = maxy; }
    void setMaxZ(coord maxz) { _maxz = maxz; }

    void rotateX(coord rx) { _rx = rx; }
    void rotateY(coord ry) { _ry = ry; }
    void rotateZ(coord rz) { _rz = rz; }

    void save();
};

#endif
