/*
 *  TriMeshOpenGLViewer-Phong
 *  Copyright (C) 2005  Thanassis Tsiodras (ttsiod@softlab.ntua.gr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "Point.h"

Point::Point(coord x, coord y, coord z)
{
    _data[0] = x; 
    _data[1] = y; 
    _data[2] = z;
    _data[3] = 1.0;
}

Point::~Point()
{
}

Point& Point::operator+=(const Point& offset)
{
    _data[0] += offset._data[0];
    _data[1] += offset._data[1];
    _data[2] += offset._data[2];

    return *this;
}

void Point::move(const Point& offset)
{
    operator+=(offset);
}

void Point::move(coord dx, coord dy, coord dz)
{
    _data[0] += dx;
    _data[1] += dy;
    _data[2] += dz;
}

void Point::set(coord x, coord y, coord z)
{
    _data[0] = x;
    _data[1] = y;
    _data[2] = z;
}

void Point::set(const Point& point)
{
    set(point._data[0], point._data[1], point._data[2]);
}

void Point::normalize()
{
    coord sum = _data[0]*_data[0] + _data[1]*_data[1] + _data[2]*_data[2];

    sum = sqrt(sum);
    if (sum>1e-5) {
        _data[0]/=sum;
        _data[1]/=sum;
        _data[2]/=sum;
    } else {
        _data[0] = 0.;
        _data[1] = 0.;
        _data[2] = 0.;
    }
}

ostream& operator<<(ostream& stream, Point& point)
{
    stream << point.getX() << ",";
    stream << point.getY() << ",";
    stream << point.getZ();
    return stream;
}
