/*
 *  TriMeshOpenGLViewer-Phong
 *  Copyright (C) 2005  Thanassis Tsiodras (ttsiod@softlab.ntua.gr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __TRIANGLE_H__
#define __TRIANGLE_H__

#include "Types.h"
#include "Exceptions.h"

class Vertex;

class Triangle {
    friend class Object;

    Vertex *_v[3];
    Point _normal;
    Point _center;
    GLfloat _ambientOcclusionColorArrayGL[4];
    GLfloat _colorArrayGL[4];
public:
    Triangle(
	Vertex *v1, Vertex *v2, Vertex *v3,
	coord r, coord g, coord b)
	:
	_normal(1e8,1e8,1e8),
	_center(1e8,1e8,1e8)
    {
        _v[0] = v1; _v[1] = v2; _v[2] = v3;
	GLfloat _ambientOcclusionFactor(
	    (v1->getAmbientOcclusionFactor() + 
	     v2->getAmbientOcclusionFactor() + 
	     v3->getAmbientOcclusionFactor())/3.0);
	_ambientOcclusionColorArrayGL[0] = _ambientOcclusionFactor*r;
	_ambientOcclusionColorArrayGL[1] = _ambientOcclusionFactor*g;
	_ambientOcclusionColorArrayGL[2] = _ambientOcclusionFactor*b;
	_ambientOcclusionColorArrayGL[3] = 1.0;
	_colorArrayGL[0] = r;
	_colorArrayGL[1] = g;
	_colorArrayGL[2] = b;
	_colorArrayGL[3] = 1.0;
    }
    Vertex *getV1() { return _v[0]; }
    Vertex *getV2() { return _v[1]; }
    Vertex *getV3() { return _v[2]; }
    void setV1(Vertex *v) { _v[0]  = v; }
    void setV2(Vertex *v) { _v[1] = v; }
    void setV3(Vertex *v) { _v[2] = v; }
    Point& getNormal() { return _normal; }
    Point& getCenter() { return _center; }
    GLfloat *getColorGL() { return &_colorArrayGL[0]; }
    GLfloat *getAmbientOcclusionColorGL() { return &_ambientOcclusionColorArrayGL[0]; }
    void updateCenter() {
        _center.set(0.,0.,0.);
        if (_v[0]) 
	    _center.move( _v[0]->getX(), _v[0]->getY(), _v[0]->getZ());
	else { THROW("_v[0] is NULL in updateCenter"); }
        if (_v[1]) 
	    _center.move( _v[1]->getX(), _v[1]->getY(), _v[1]->getZ());
	else { THROW("_v[1] is NULL in updateCenter"); }
        if (_v[2]) 
	    _center.move( _v[2]->getX(), _v[2]->getY(), _v[2]->getZ());
	else { THROW("_v[2] is NULL in updateCenter"); }
        _center.set( _center.getX()/3, _center.getY()/3, _center.getZ()/3);
    }
};

#endif
