/*
 *  renderer - A simple implementation of polygon-based 3D algorithms.
 *  Copyright (C) 2004  Thanassis Tsiodras (ttsiodras@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __types_h__
#define __types_h__

#ifdef _WIN32
#include <windows.h>
#include <GL/gl.h>
#endif

#include <SDL.h>
#include <math.h>

typedef enum {
    RENDER_POINTS=1024,
    RENDER_LINES,
    RENDER_GOURAUD,
    RENDER_PHONG
} RenderState;

#ifdef _WIN32
typedef ptrdiff_t GLsizeiptrARB;
typedef void APIENTRY (*GlGenProgramsARB)(GLsizei, GLuint *);
typedef void APIENTRY (*GlBindProgramARB)(GLenum, GLuint);
typedef void APIENTRY (*GlProgramStringARB)(GLenum, GLenum, GLsizei, const GLvoid *);
typedef void APIENTRY (*GlProgramEnvParameter4fvARB)(GLenum, GLuint, const GLfloat *);
typedef void APIENTRY (*GlGenBuffersARB)(GLsizei, GLuint *);
typedef void APIENTRY (*GlBindBufferARB)(GLenum, GLuint);
typedef void APIENTRY (*GlBufferDataARB)(GLenum, GLsizeiptrARB, const GLvoid *, GLenum);
#endif

typedef float coord;

struct Vector3 
{
    coord _x, _y, _z;

    Vector3(coord x=0, coord y=0, coord z=0)
	:
	_x(x), _y(y), _z(z) {}

    Vector3(const Vector3& rhs)
	:
	_x(rhs._x),
	_y(rhs._y),
	_z(rhs._z) {}

    void normalize() 
    {
	coord norm = sqrt(_x*_x +_y*_y + _z*_z);
	_x /= norm;
	_y /= norm;
	_z /= norm;
    }

    Vector3& operator+=(const Vector3& rhs)
    {
	_x += rhs._x;
	_y += rhs._y;
	_z += rhs._z;
	return *this;
    }
    
    Vector3& operator-=(const Vector3& rhs)
    {
	_x -= rhs._x;
	_y -= rhs._y;
	_z -= rhs._z;
	return *this;
    }
    
    Vector3& operator*=(const coord& rhs)
    {
	_x *= rhs;
	_y *= rhs;
	_z *= rhs;
	return *this;
    }
    
    Vector3& operator/=(const coord& rhs)
    {
	_x /= rhs;
	_y /= rhs;
	_z /= rhs;
	return *this;
    }
};

#endif
