/*
 *  TriMeshOpenGLViewer-Phong
 *  Copyright (C) 2005  Thanassis Tsiodras (ttsiod@softlab.ntua.gr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <math.h>
#include <assert.h>

#include "Defines.h"
#include "Vertex.h"
#include "Object.h"
#include "Light.h"
#include "Variables.h"
#include "Exceptions.h"

#define PACK_ANGLE		28
#define VERTEX_POOL_SIZE	16384

int Vertex::findBreakingNormal(Object *parent)
{
    int VertexArrayChanged = 0;
again:
    assert(parent);
    std::vector<Normal *>::iterator nrm,nrm1;
    nrm = _normals.begin();

    if(nrm == _normals.end()) { // No triangle points to this vertex!
        return -1;              // Return and delete it...
    }
    _normal.set(0.,0.,0.);
    while(nrm != _normals.end()) {
        _normal.move(*(*nrm));
        nrm++;
    }
    _normal.normalize();

    for(nrm = _normals.begin(); nrm != _normals.end(); nrm++) {
        coord dp =
            _normal.getX() * (*nrm)->getX() +
            _normal.getY() * (*nrm)->getY() +
            _normal.getZ() * (*nrm)->getZ();
        coord angle = coord(180*acos((double)dp)/M_PI);
        if (fabsf(angle) > g_interAngle) {

            // This contributing normal is very different from the
            // average normal of this vertex. The normal has been
            // contributed from a vertex that should not be shared.
            Normal *n = *nrm;
            Vertex *nv = new Vertex(getX(),getY(),getZ(),parent,0.,0.,0.);
            *((*nrm)->getOrigin()) = nv;
            nv->addNormal(*nrm);
            parent->addVertex(nv);
            _normals.erase(nrm);

            // Pack together all the normals that look like this one
            // in the new vertex.
            for(nrm1 = _normals.begin(); nrm1 != _normals.end();) {
                coord dp =
                    n->getX() * (*nrm1)->getX() +
                    n->getY() * (*nrm1)->getY() +
                    n->getZ() * (*nrm1)->getZ();
                coord angle = coord(180*acos((double)dp)/M_PI);
                if (fabsf(angle) < g_interAngle) {
                    *((*nrm1)->getOrigin()) = nv;
                    nv->addNormal(*nrm1);
                    nrm1 = _normals.erase(nrm1);

                } else
                    nrm1++;
            }
            VertexArrayChanged = 1;

            // Since the Vertex normal array has changed,
            // the avg normal must be recomputed...
            goto again;
        }
    }
    return VertexArrayChanged;
}

