/*
 *  TriMeshOpenGLViewer-Phong
 *  Copyright (C) 2005  Thanassis Tsiodras (ttsiod@softlab.ntua.gr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __VERTEX_H__
#define __VERTEX_H__

#include <vector>
#include "Types.h"
#include "Point.h"
#include "Object.h"
#include "Screen3D.h"
#include "Normal.h"

class Triangle;
class Light;

class Vertex : public Point {
    Object *_object;
    Point _normal;
    std::vector<Normal*> _normals;
    GLfloat _ambientOcclusionFactorArrayGL[4];
public:
    Vertex(
	coord x, coord y, coord z, 
	Object *object, 
	coord nx, coord ny, coord nz, 
	coord ambientOcclusionFactor = 0.25)
	:
	Point(x,y,z),
	_object(object),
	_normal(nx,ny,nz)
    {
	_ambientOcclusionFactorArrayGL[0] = ambientOcclusionFactor;
	_ambientOcclusionFactorArrayGL[1] = ambientOcclusionFactor;
	_ambientOcclusionFactorArrayGL[2] = ambientOcclusionFactor;
	_ambientOcclusionFactorArrayGL[3] = 1.0;
        if (object->getMinX() > x) object->setMinX(x);
        if (object->getMinY() > y) object->setMinY(y);
        if (object->getMinZ() > z) object->setMinZ(z);
        if (object->getMaxX() < x) object->setMaxX(x);
        if (object->getMaxY() < y) object->setMaxY(y);
        if (object->getMaxZ() < z) object->setMaxZ(z);
    }

    GLfloat getAmbientOcclusionFactor() { 
	return _ambientOcclusionFactorArrayGL[0]; 
    }

    GLfloat *getAmbientOcclusionFactorArrayGL() { 
	return &_ambientOcclusionFactorArrayGL[0]; 
    }

    void set(coord x, coord y, coord z)
    {
        Point::set(x,y,z);
        if (_object->getMinX() > x) _object->setMinX(x);
        if (_object->getMinY() > y) _object->setMinY(y);
        if (_object->getMinZ() > z) _object->setMinZ(z);
        if (_object->getMaxX() < x) _object->setMaxX(x);
        if (_object->getMaxY() < y) _object->setMaxY(y);
        if (_object->getMaxZ() < z) _object->setMaxZ(z);
    }

    Object *getObject() { return _object; }
    void setObject(Object *obj) { _object = obj; }

    Point* getNormal() { return &_normal; }
    void addNormal(Normal *nrm) { _normals.push_back(nrm); }
    int findBreakingNormal(Object *);
    std::vector<Normal*>& getNormals() { return _normals; }

    void illuminatePhong(Light *, Camera&);
};

#endif
